
import UIKit

enum RecommendationTempleteType: String, Codable {
    case productSuggestion = "ProductSuggestion"
}

class TaskRecommendationTemplate: Codable {
    
    var templateType: RecommendationTempleteType
    var title: String?
    var items: [TaskRecommendationItem] = []
    
    private enum CodingKeys: String, CodingKey {
        case templateType
        case title
        case items
    }
    
    enum InfoTypeKey: CodingKey {
        case templateType
    }
    
    required init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
        templateType = try container.decode(RecommendationTempleteType.self, forKey: .templateType)
        title = try container.decodeIfPresent(String.self, forKey: .title)
        
        var itemsContainer = try container.nestedUnkeyedContainer(forKey: .items)
        var itemsArray = [TaskRecommendationItem]()
        while !itemsContainer.isAtEnd {
            switch templateType {
            case .productSuggestion:
                let product = try itemsContainer.decode(TaskRecommendationItemProductSuggestion.self)
                itemsArray.append(product)
            }
        }
        items = itemsArray
    }
}

class TaskRecommendationItem: Codable {
    var title: String?
    
    private enum CodingKeys: String, CodingKey {
        case title
    }
}

class TaskRecommendationItemProductSuggestion: TaskRecommendationItem {
    var price: Int
    var previewImage: String
    var url: URL?
    
    private enum CodingKeys: String, CodingKey {
        case price
        case previewImage
        case url
    }
    
    required init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
        url = try container.decode(URL.self, forKey: .url)
        price = try container.decode(Int.self, forKey: .price)
        previewImage = try container.decode(String.self, forKey: .previewImage)
        try super.init(from: decoder)
    }
}

